using System;
using System.Net.Sockets;
using System.Net;
using System.IO;
using System.Text;
using System.Threading;

namespace MediaCenter.DigitalHome
{
	/// <summary>
	/// Connection to the server used to receive binary data.
	/// </summary>
	public class DownloadConnection : Connection
	{		
		/* Buffer variables */
		private static readonly int BUFFER_SIZE = 1024;
		private byte[] buffer;
		
		private TcpListener listener;
		private string completeFilePath;
		private bool waitingForData;
		FileTransferManager ftMan;
		
		/// <summary>
		/// Constructor. 
		/// </summary>
		/// <param name="client">The Client holding this DownloadConnection</param>
		public DownloadConnection(DigitalHomeClient client, FileTransferManager ftMan) : base(client)
		{
			listener = new TcpListener(NetworkVariables.DOWNLOAD_PORT);
			buffer = new byte[BUFFER_SIZE];
			this.ftMan = ftMan;

			completeFilePath = null;
			waitingForData = true;
		}
		
		/// <summary>
		/// Sets the complete path (path + file name + extention) of the
		/// file in which to store the received data.
		/// </summary>
		/// <param name="completeFilePath">Path + file name + extention</param>
		public void setCompleteFilePath(string completeFilePath)
		{
			this.completeFilePath = completeFilePath;
		}
		
		/// <summary>
		/// Method used to reset the DownloadConnection
		/// after a file has been received.
		/// </summary>
		public void reset()
		{
			socket.Close();
			completeFilePath = null;
			waitingForData = false;
		}
		
		/// <summary>
		/// The main thread method.
		/// Creates a socket and listens for connections. When a connection is made
		/// it waits for data do be sent. Received data is stored in a new file
		/// determined by the completeFilePath.
		/// </summary>
		public override void run()
		{
			while(running) 
			{
				client.log("Lytter etter ny downloadrequest");
				/* Starts listening for connections */
				listener.Start();
				socket = listener.AcceptTcpClient();
				waitingForData = true;
			
				while(true)
				{
					try 
					{
						long lastActivity;
						int timeout;
						FileStream file;
						BufferedStream bufferedFileStream;
						string fileName;
						string extension;

						/* A connection has been made */
						stream = socket.GetStream();
					
						/* Loops as long as no data is received */
						while(!stream.DataAvailable)
						{
							Thread.Sleep(500);
							
							/* Checks if it is expected to receive
							 * data on this stream. This check makes it possible to 
							 * reset the download connection. Breaks the 
							 * while(!stream.DataAvailable) and returns to the while(true) */
							if (!waitingForData) 
							{
								break;
							}
						}
						
						/* No data is expected. Breaks the while(true) and returns to
						 * waiting for a connection */
						if (!waitingForData) 
						{
							break;
						}
						
						/* if the colpleteFilePath is null, it will not be possible
						 * to save the received data */
						if(null == completeFilePath)
						{
							client.log("filePath er null");
							break;
						}

						file = new FileStream(completeFilePath, FileMode.CreateNew, FileAccess.Write);

						bufferedFileStream = new BufferedStream(file);
						
						/* Timestamp used to check for timeout */
						lastActivity = DateTime.Now.Ticks;
						timeout = (int)TimeSpan.FromSeconds(30).TotalSeconds;

						while(true)
						{
							/* Data is available and is stored in the file */
							if(stream.DataAvailable) 
							{
								int len = stream.Read(buffer, 0, BUFFER_SIZE);

								if(len > 0)
								{
									bufferedFileStream.Write(buffer, 0, len);
								}
								/* Timestamps the last recived data */
								lastActivity = DateTime.Now.Ticks;			
							}
								/* No data is received */
							else 
							{
								/* This code does not perform a check to see if
								 * the entire file has been received. If no data is received
								 * within a timeout value, we assume the entire file is received
								 * */
								if(DateTime.Now.Ticks > lastActivity + ((long)(timeout)) * 10000)
								{	
									client.log(completeFilePath +  " ferdig nedlastet");
									break;
								}
							}					
						}
						
						/* flushes and closes the streams and the file */
						bufferedFileStream.Flush();
						bufferedFileStream.Close();
						file.Close();
				
						/* Sends a "file OK" message to the server */
						fileName = Path.GetFileNameWithoutExtension(completeFilePath);
						extension = Path.GetExtension(completeFilePath);
						ftMan.sendDownloadFileOk(fileName, extension);

						completeFilePath = null;
					}
					catch (Exception e)
					{
						client.log(e.StackTrace);
					}
				}
			}
		}	
		
		/// <summary>
		/// Closes the download connection
		/// </summary>
		public void closeConnection()
		{
			listener.Stop();
			close();
		}
	}
}
