using System;
using System.IO;
using System.Threading;
using System.Xml;
using System.Xml.XPath;
using System.Diagnostics;

namespace MediaCenter.DigitalHome
{
	/// <summary>
	/// Manages file transfer to/from the server
	/// </summary>
	public class FileTransferManager
	{
		/* Static strings */
		private static readonly string DIGITAL_HOME_APP_DATA_FOLDER =
			Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) +
			"\\DigitalHome";
		private static readonly string UPDATE_UTIL_NAME = "gacutil";
		private static readonly string UPDATE_UTIL_TYPE = "exe"; 
		private static readonly string SKIP_FILE = "skipFile";
		private static readonly string UPDATE_FILE_NAME = "DigitalHomeClient";
		private static readonly string UPDATE_FILE_TYPE = "dll";
		
		private XMLConnection xmlCon;
		private DownloadConnection downCon;
		private UploadConnection upCon;
		private DigitalHomeClient client;
		private FileTypeResolver fileTypeResolver;
		private bool newUpdate;
		
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="client">The Client holding this DownloadManager</param>
		/// <param name="xmlCon">The Client's xmlConnection</param>
		public FileTransferManager(DigitalHomeClient client, XMLConnection xmlCon)
		{
			this.client = client;
			this.xmlCon = xmlCon;
			fileTypeResolver = new FileTypeResolver();
			newUpdate = false;
			
			downCon = new DownloadConnection(client, this);
			Thread t1 = new Thread(new ThreadStart(downCon.run));
			t1.IsBackground = true;
			t1.Start();
			
            startNewUploadConnection();			
		}
		
		/// <summary>
		/// Replaces the UploadConnection with a new one.
		/// </summary>
		public void startNewUploadConnection() 
		{
			upCon = new UploadConnection(client, this);
			Thread t2 = new Thread(new ThreadStart(upCon.run));
			t2.IsBackground = true;
			t2.Start();
		}

		/// <summary>
		/// Before a file is downloaded, the server send an XML document
		/// containing the file -name and -type. The DownloadManager uses
		/// this information to store the file correctly
		/// </summary>
		/// <param name="fileName">The name of the next file to be downloaded</param>
		/// <param name="fileType">The type of the next file to be downloaded</param>
		public void fileDownloadInfoReceived(string fileName, string fileType)
		{
			string filePath = resolveDownloadDestination(fileName, fileType);
			
			if (SKIP_FILE.Equals(filePath))
			{
				sendDownloadFileOk(fileName, fileType);
			}
			else 
			{

				/* If the file is not an update file: 
				 * Checks if the file name exist. Names the file fileName(n) if it does.
				 * */
				if(!newUpdate)
				{
					int n = 1;
					string orgFileName = fileName;  

					while(File.Exists(createPath(filePath, fileName, fileType)))
					{
						fileName = orgFileName+ "(" + n + ")";
						n++;
					}
				}
			
				/* The file is received on the downloadConnection.
				 * For the downloadConnection to know where to store
				 * the received file, and with what name, the fileinfo is stored
				 * at the downloadConnection */ 
				downCon.setCompleteFilePath(createPath(filePath, fileName, fileType));
			
				/* File info received. Sends a message to the server
				 * asking to send the actual file */
				sendDownloadFileInfoReceived();
			}
		}
		
		/// <summary>
		/// The server sends status messages to the DownloadManager. This
		/// method receives them and perform the appropriate action.
		/// </summary>
		/// <param name="status">The status message</param>
		public void downloadStatusReceived(string status)
		{
			/* As of now, only CLOSE messages are sent from the server */
			if(XMLVariables.STATUS_CLOSE.Equals(status))
			{
				downCon.reset();
			}
		}
		
		/// <summary>
		/// Resolves where to store a file based on its file type
		/// </summary>
		/// <param name="fileType">The extention of the file</param>
		/// <returns>Path where the file shoud be stored</returns>
		private string resolveDownloadDestination(string fileName, string fileType)
		{
			/* Gets the path of My Documents */
			string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal);

			/* Changes the path depending on the fileType */
			if (UPDATE_UTIL_NAME.Equals(fileName) && UPDATE_UTIL_TYPE.Equals(fileType))
			{
				if (File.Exists(createPath(DIGITAL_HOME_APP_DATA_FOLDER, fileName, fileType)))
				{
					path = SKIP_FILE;
				}
				else 
				{
					path = DIGITAL_HOME_APP_DATA_FOLDER;
				}
			}
			else if (UPDATE_FILE_NAME.Equals(fileName) && UPDATE_FILE_TYPE.Equals(fileType))
			{
				path = DIGITAL_HOME_APP_DATA_FOLDER;
				newUpdate = true;
			}
			else if (fileTypeResolver.isAudio(fileType))
			{
				path = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
			} 
			else if (fileTypeResolver.isPicture(fileType))
			{
				path = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
			}
			else if (fileTypeResolver.isVideo(fileType))
			{
				// path = TODO: fikse en hack for  legge inn i My Videos / Mine videoer
			}
			
			/* Checks if the path exists and creates is if it does not */
			if (!Directory.Exists(path))
			{
				Directory.CreateDirectory(path);
			}

			return path;
		}
		
		/// <summary>
		/// Creates an XML document stating that the file info 
		/// has been received successfully. This document is passed 
		/// on to the XMLConnection to be sent to the server
		/// </summary>
		private void sendDownloadFileInfoReceived()
		{
			XmlDocument doc = new XmlDocument();

			XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
			XmlElement root = doc.CreateElement(XMLVariables.SERVICE_ROOT);
			root.SetAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_DOWNLOAD_ID);

			XmlElement body = doc.CreateElement(XMLVariables.SERVICE_BODY);
			XmlElement status = doc.CreateElement(XMLVariables.STATUS);
			XmlText statusText = doc.CreateTextNode(XMLVariables.STATUS_FILEINFO_OK);

			status.AppendChild(statusText);
			body.AppendChild(status);
			root.AppendChild(body);
			
			doc.AppendChild(xmlDecl);
			doc.AppendChild(root);

			xmlCon.sendXMLDoc(doc);
		}

		/// <summary>
		/// Creates an XML document stating that the file 
		/// has been received successfully. This document is passed 
		/// on to the XMLConnection to be sent to the server
		/// </summary>
		public void sendDownloadFileOk(String fileName, String extension)
		{	

			if(newUpdate)
			{	
				Process proc = new Process();
				string title;
				string text;
				
				/* Installs the new update */
				proc.StartInfo.FileName = 
					"\"" + createPath(DIGITAL_HOME_APP_DATA_FOLDER, UPDATE_UTIL_NAME, UPDATE_UTIL_TYPE) + "\"";
				proc.StartInfo.Arguments = "-i " +
					"\"" + createPath(DIGITAL_HOME_APP_DATA_FOLDER, UPDATE_FILE_NAME, UPDATE_FILE_TYPE) + "\"";
				proc.StartInfo.UseShellExecute = false;
				proc.StartInfo.RedirectStandardOutput = true;
				proc.StartInfo.CreateNoWindow= true;
				proc.Start();
				proc.WaitForExit();
                
				newUpdate = false;
				
				/* Removes the update file */
				System.IO.File.Delete(createPath(DIGITAL_HOME_APP_DATA_FOLDER, UPDATE_FILE_NAME, UPDATE_FILE_TYPE));

				title = "Melding fra Tjenesteleverandr: ";
				text = "Din klient har blitt oppdatert. \r\nEndringer trer i kraft neste gang Media Centeret startes opp.";
				client.displayPopUp(title, text);
			}

			XmlDocument doc = new XmlDocument();

			XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
			XmlElement root = doc.CreateElement(XMLVariables.SERVICE_ROOT);
			root.SetAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_DOWNLOAD_ID);
			XmlElement body = doc.CreateElement(XMLVariables.SERVICE_BODY);

			XmlElement status = doc.CreateElement(XMLVariables.STATUS);
			XmlText statusText = doc.CreateTextNode(XMLVariables.STATUS_FILE_OK);
			status.AppendChild(statusText);

			XmlElement fileNameElement = doc.CreateElement(XMLVariables.FILE_NAME);
			XmlText fileNameText = doc.CreateTextNode(fileName);
			fileNameElement.AppendChild(fileNameText);

			XmlElement fileTypeElement = doc.CreateElement(XMLVariables.FILE_TYPE);
			XmlText fileTypeText = doc.CreateTextNode(extension);
			fileTypeElement.AppendChild(fileTypeText);
			
			body.AppendChild(status);
			body.AppendChild(fileNameElement);
			body.AppendChild(fileTypeElement);
			root.AppendChild(body);
			doc.AppendChild(xmlDecl);
			doc.AppendChild(root);

			xmlCon.sendXMLDoc(doc);
		}
		
		/// <summary>
		/// Puts the complete path of a file into one string
		/// </summary>
		/// <param name="folder">The path of the folder</param>
		/// <param name="fileName">The file name</param>
		/// <param name="fileType">The file's extention</param>
		/// <returns></returns>
		private string createPath(string folder, string fileName, string fileType)
		{
			return folder + "\\" + fileName + "." + fileType;
		}
		
		/// <summary>
		/// Handles incoming upload request's
		/// </summary>
		/// <param name="path">The path of the file</param>
		/// <param name="fileName">The file name</param>
		/// <param name="fileType">The file's extention</param>
		public void fileUploadRequestReceived(string path, string fileName, string fileType)
		{			
			/* Adds the request to the upload connection's queue */
			upCon.addRequest(createPath(path, fileName, fileType));
			
			/* Starts transfering if no transfer is currently running */
			if(!upCon.isRunning())
			{
				client.log("FileTransferManager: ingen uploads i k, s starter ny sending");
				upCon.startTransfer();
			}
		}
		
		/// <summary>
		/// Handles upload status messages 
		/// </summary>
		/// <param name="status"></param>
		public void uploadStatusReceived(string status)
		{
			/* FileInfoOk received. Starts transfering the file */
			if(XMLVariables.STATUS_FILEINFO_OK.Equals(status))
			{
				upCon.activateFileTransfer();
			}
			/* FileOk received */
			else if(XMLVariables.STATUS_FILE_OK.Equals(status))
			{
				upCon.fileOkRecieved();
			}
			/* FileFailed received */
			else if(XMLVariables.STATUS_FILE_FAILED.Equals(status))
			{
				/* Here we are just informing the download connection to remote the element from the queue
				 * and start transfering the next element. Some sort of retransmission of the failed element 
				 * could be implemented here. */
				upCon.fileOkRecieved();
			}
		}
		
		/// <summary>
		/// Closes the FileTransferManager
		/// </summary>
		public void close() 
		{
			downCon.closeConnection();
			upCon.closeConnection();
		}
	}
}
