using System;
using System.IO;
using System.Collections;
using System.Xml;
using System.Threading;
using System.Net.Sockets;
using System.Net;

namespace MediaCenter.DigitalHome
{
	/// <summary>
	/// Connection to the server used to send binary data.
	/// </summary>
	public class UploadConnection : Connection
	{
		/* File buffer */
		private static readonly int BUFFER_SIZE = 1024;
		private byte[] buffer;

		private TcpListener listener;
		private ArrayList queue;
		private FileTransferManager ftMan;
		private XMLConnection xmlCon;
		
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="client">The Client holding this UploadConnection</param>
		/// <param name="ftMan">The FileTransferManager holding this UploadConnection</param>
		public UploadConnection(DigitalHomeClient client, FileTransferManager ftMan) : base(client)
		{
			buffer = new byte[BUFFER_SIZE];
			queue = new ArrayList();
			this.ftMan = ftMan;
			xmlCon = client.getXmlConnection();
			running = false;
			
			listener = new TcpListener(NetworkVariables.UPLOAD_PORT);
		}
		
		/// <summary>
		/// Listens for connections until a connection is made. 
		/// To start listening for a new connection, create a fresh UploadConnection
		/// </summary>
		public override void run()
		{			
			listener.Start();				
			socket = listener.AcceptTcpClient();
			stream = socket.GetStream();
			listener.Stop();
		}
		
		/// <summary>
		/// Adds an upload request to the queue
		/// </summary>
		/// <param name="completePath">The complete path (path\fileName.extention) of the file to be uploaded</param>
		public void addRequest(string completePath)
		{
			queue.Add(completePath);
		}
		
		/// <summary>
		/// Sends a file info message to the server. 
		/// This tells the server to make ready to receive the file.
		/// </summary>
		/// <param name="completePath">The complete path (path\fileName.extention) of the file to be uploaded</param>
		private void sendFileInfo(string completePath)
		{
			string[] fileInfo = FileInfoTokenizer.tokenize(completePath);
			
			XmlDocument doc = new XmlDocument();			
			XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
			XmlElement root = doc.CreateElement(XMLVariables.SERVICE_ROOT);
			root.SetAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_UPLOAD_ID);
			XmlElement body = doc.CreateElement(XMLVariables.SERVICE_BODY);
			XmlElement fileType = doc.CreateElement(XMLVariables.FILE_TYPE);
			XmlElement fileName = doc.CreateElement(XMLVariables.FILE_NAME);
			XmlText fileTypeText = doc.CreateTextNode(fileInfo[2]);
			XmlText fileNameText = doc.CreateTextNode(fileInfo[1]);
		
			fileType.AppendChild(fileTypeText);
			fileName.AppendChild(fileNameText);
			body.AppendChild(fileType);
			body.AppendChild(fileName);
			root.AppendChild(body);
			doc.AppendChild(xmlDecl);
			doc.AppendChild(root);

			xmlCon.sendXMLDoc(doc);
		}
		
		/// <summary>
		/// Sends a close message to the server, closes this connection and replases
		/// the file transfer manager's upload connection with a new one.
		/// </summary>
		private void sendClose()
		{
			sendStatusMessage(XMLVariables.STATUS_CLOSE);
			close();
			ftMan.startNewUploadConnection();
			
		}
		
		/// <summary>
		/// Sends a status message to the server
		/// </summary>
		/// <param name="statusText">The status text</param>
		private void sendStatusMessage(string statusText)
		{
			XmlDocument doc = new XmlDocument();			
			XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
			XmlElement root = doc.CreateElement(XMLVariables.SERVICE_ROOT);
			root.SetAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_UPLOAD_ID);
			XmlElement body = doc.CreateElement(XMLVariables.SERVICE_BODY);
			XmlElement status = doc.CreateElement(XMLVariables.STATUS);
			XmlText txtStatus = doc.CreateTextNode(statusText);
			
			status.AppendChild(txtStatus);
			body.AppendChild(status);
			root.AppendChild(body);
			doc.AppendChild(xmlDecl);
			doc.AppendChild(root);

			xmlCon.sendXMLDoc(doc);
		}

		/// <summary>
		/// Sends the firs file in queue to the server
		/// </summary>
		private void uploadFile()
		{	
			/* If the queue is empty, no file is sent */
			if(queue.Count == 0)
			{
				return;
			}

			string completePath = (string)queue[0];
			
			try 
			{				
				/* Makes ready to read the file */
				client.log("UploadConnection: Prver  sende fil: "+completePath);
				FileStream file = new FileStream(completePath, FileMode.Open, FileAccess.Read);
				BufferedStream bufferedIn = new BufferedStream(file);
				int len = 0;
				
				/* Reads the file into buffer and sends it to the server */
				while( (len = bufferedIn.Read(buffer, 0, BUFFER_SIZE)) > 0)
				{
					stream.Write(buffer, 0, len);
				}
				
				bufferedIn.Close();
			}
			catch (Exception e) 
			{
				client.log("UploadConnection: Filsending feilet");
				client.log(e.StackTrace);
				
				closeConnection();
			}
		}

		/// <summary>
		/// Starts a new transfer by sending file info to the server.
		/// If, of some reason, the file does not exist, a status message
		/// of this is sent.
		/// </summary>
		public void startTransfer() 
		{
			string completePath = (string)queue[0];

			running = true;
			
			/* The file does not exist */
			if(!File.Exists(completePath))
			{
				sendStatusMessage(XMLVariables.STATUS_NO_FILE);
			} 
			/* Everything is ok. Sending file info */
			else 
			{
				sendFileInfo(completePath);
			}
		}
		
		/// <summary>
		/// Transfers the first file in queue, as a new thread.
		/// </summary>
		public void activateFileTransfer()
		{			
			Thread t = new Thread(new ThreadStart(this.uploadFile));
			t.IsBackground = true;
			t.Start();
		}
		
		/// <summary>
		/// A fileOk has been received from the server.
		/// If there is more requests in queue, these are processed.
		/// If not, a close message is sent to the server.
		/// </summary>
		public void fileOkRecieved()
		{
			queue.Remove(queue[0]);
		
			if (queue.Count > 0) 
			{
				startTransfer();
			} 
			else 
			{
				sendClose();
			}
		}
		
		/// <summary>
		/// Closes this connection
		/// </summary>
		public void closeConnection()
		{
			this.close();
		}
	}
}
