using System;
using System.Xml;
using System.Xml.XPath;
using System.Net.Sockets;
using System.Net;
using System.IO;
using System.Text;
using System.Threading;
using MediaCenter.DigitalHome;

namespace MediaCenter.DigitalHome
{
	/// <summary>
	/// Connection to the server for sending and 
	/// receiving XML documents.
	/// </summary>
	public class XMLConnection : Connection
	{
		private bool authed = false;
		
		/// <summary>
		/// Constructor. Opens a connection to the server
		/// </summary>
		/// <param name="client">The Client holding this XMLConnection</param>
		public XMLConnection(DigitalHomeClient client) : base(client)
		{
			try 
			{
				socket = new TcpClient(NetworkVariables.SERVER_URL, NetworkVariables.SERVER_PORT);
				stream = socket.GetStream();
			}
			catch (Exception e) 
			{
				client.displayPopUp("Connection error", "Unable to connect to service provider server");
			}
		}
		
		/// <summary>
		/// Controls the authed property.
		/// If the XMLConnection is not authed, only auth messages may be sent
		/// </summary>
		/// <param name="auth">true/false</param>
		public void setAuthed(bool auth)
		{
			authed = auth;
		}
		
		/// <summary>
		/// Gets the authed property
		/// </summary>
		/// <returns>If this XMLConnection is authed or not</returns>
		public bool isAuthed()
		{
			return authed;
		}

		/// <summary>
		/// The method to be run by a thread.
		/// Listens to the stream and passes XML document to the 
		/// Client when received
		/// </summary>
		public override void run() 
		{
			StreamReader strReader = new StreamReader(stream);
			
			while(running)
			{
				try 
				{
					XmlDocument doc;
					
					/* The XML documents sent by the server consist of
					 * two lines, one containing the XML declaration
					 * and one containing the rest of the document */
					string returnData = strReader.ReadLine();
					returnData += strReader.ReadLine();
					
					client.log("Returndata er: "+ returnData);
					
					/* Occurs if the server sends empty data.
					 * This should not occur  */
					if("".Equals(returnData))
					{
						client.log("Mottok tom data!");
						continue;
					}

					/* Builds an XML document of the received data and 
					 * send it to the Client */
					doc = new XmlDocument();			
					doc.LoadXml(returnData);
					
					client.receiveXMLDoc(doc);
				}
				catch (Exception e) 
				{
					client.displayPopUp("Digital Home", "Mistet oppkoblingen til Service Provider. Vennligst start Media Centeret p nytt for  opprette ny forbindelse");
					client.log(e.StackTrace);
				}		
			}
		}
		
		/// <summary>
		/// Sends an authentication message to the server.
		/// Unless the XML connection is authed, no other
		/// messages can be sent
		/// </summary>
		/// <param name="username">Username</param>
		/// <param name="password">Password</param>
		public void sendAuthMessage(string username, string password)
		{
			XmlDocument doc = new XmlDocument();	
			XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
			XmlElement root = doc.CreateElement(XMLVariables.AUTH_ROOT);
			
			root.SetAttribute(XMLVariables.AUTH_USERNAME, username);
			root.SetAttribute(XMLVariables.AUTH_PASSWORD, password);
			root.SetAttribute(XMLVariables.AUTH_VERSION, client.getClientVersion());
			
			doc.AppendChild(xmlDecl);
			doc.AppendChild(root);
			
			sendAuthed(doc);
		}

		/// <summary>
		/// Method used to send XML documents to the server
		/// </summary>
		/// <param name="doc">The document to be sent</param>
		/// <returns>false if the XML connection is not authed</returns>
		public bool sendXMLDoc(XmlDocument doc) 
		{
			client.log("prver  sende dokument");
			if (!authed)
			{
				client.log("oppkoblingen er ikke autentisert!");
				return false;
			}
			else
			{
				client.log("oppkobling er autentisert, s sender melding");
				sendAuthed(doc);
				return true;
			}
		}		
		
		/// <summary>
		/// Internal method used to messages to the server.
		/// An auth check should be performed before this method is used
		/// </summary>
		/// <param name="doc">The XML document to be sent</param>
		private void sendAuthed(XmlDocument doc) 
		{
			/* Adding an EndOfFile to the stream.
			 * This is used by the JDOM parser to detect the end of the transmition 
			 * */
			string xmlFile = doc.OuterXml;

			Byte[] sendBytes = Encoding.ASCII.GetBytes(xmlFile + "\r\n");
			Byte[] bytesFinal = new Byte[sendBytes.Length+1];

			for (int i = 0; i < bytesFinal.Length-1; i++) 
			{
				bytesFinal[i] = sendBytes[i];
			}
			bytesFinal[sendBytes.Length] = 4;
			
			/* Sending the modified stream */
			stream.Write(bytesFinal,0, bytesFinal.Length);
		}
	}
}
