package digital_home;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

/**
 * Abstract class representing a connection.
 * 
 * Every extension of this class must redefine the run method for
 * receiving messages through the socket. Extending classes must also
 * implement a system for sending data through the socket.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public abstract class Connection implements Runnable{

	/* Variable telling if the connection is running */
	protected boolean running = true;
	
	/* Variable telling if the connection is authenticated */
	protected boolean authenticated = false;
	
	/* The user name assositated with this connection */
	protected String username;
	
	/* Network specific objects */
	protected Socket socket;
	protected OutputStream out;
	protected InputStream in;
	
	/* The server this connection belongs to */
	protected Server server;

	/**
	 * Creates a new connection.
	 * 
	 * @param server The server this connection belongs to.
	 * @param s The socket which this connection uses to communicate.
	 */
	public Connection(Server server, Socket socket) {
		this.server = server;
		this.socket = socket;
		
		 try {
		      out = socket.getOutputStream();
			  in = socket.getInputStream();
		    }
		    catch (IOException ex) {
		    	server.log("Connection: Error getting input and output streams from socket");
		    }
	}
	
	/**
	 * Sets the connection as authenticated as well as providing the
	 * user name assosiated with this connection.
	 * 
	 * @param username The user name assosiated with this connection.
	 */
	public void autenticate(String username) {
		authenticated = true;
		this.username = username;
	}
	
	/**
	 * Checks if this connection is authenticated.
	 * 
	 * @return true if the connection is authenticated, else false.
	 */
	public boolean isAuthenticated() {
		return authenticated;
	}
	
	/**
	 * Retrieves the user name assosiated with this connection.
	 * 
	 * @return the user name assosiated with this connection.
	 */
	public String getUsername() {
		return username;
	}
	
	/**
	 * Returns the InetAddress from the socket this connection uses.
	 * 
	 * @return the InetAddress from the socket.
	 */
	public InetAddress getInetAddress() {
		return socket.getInetAddress();
	}
	
	/**
	 * The username of the connection is used to represent the connection as a String. 
	 */
	public String toString() {
		return username;
	}
	
	/**
	 * This method defines the listening behaviour of the connection and
	 * take appropriate actions when the connection receives data through
	 * the socket.
	 * 
	 * This method must be defined for each subclass inhereting from this class.
	 */
	public abstract void run() ;
	
	/**
	 * Abstract method. Defines the close procedure of a connection. 
	 */
	public abstract void close();

}
