package digital_home;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This class is a connection manager which listenes for incoming connections 
 * and manages all connections used to communicate with clients through
 * XML documents.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class ConnectionManager implements Runnable{
	
	/* Variable telling if the connection manager is running */
	private boolean running = true;
	
	/* The server socket this connection manager is listening to */
	private ServerSocket listener;
	
	/* List of authenticated and not authenticated connections */
	private Hashtable<String, XMLConnection> authConnections;
	private ArrayList<XMLConnection> connections;
	
	/* The server this connection manager belings to */
	private Server server;
	
	/**
	 * Creates the connection manager and server socket.
	 * 
	 * @param server The server this connection manager belongs to.
	 */
	public ConnectionManager(Server server) {
		this.server = server;
		connections = new ArrayList<XMLConnection>();
		authConnections = new Hashtable<String, XMLConnection>();
		
		try {
			listener = new ServerSocket(NetworkVariables.CON_MAN_PORT);
		} catch (IOException e) {
			server.log("ConnectionManager: Failed to intialize server socket.");
		}
	}

	/**
	 * Method for listening for new connections. New connections are added to 
	 * the connection list for unauthenticated connections.
	 */
	public void run() {
		
		while (running) {
			try {
				Socket s = listener.accept();
				XMLConnection con = new XMLConnection(server, s);
				
				connections.add(con);
				server.addNonAuthedConnection(con);
				Thread t = new Thread(con);
				t.start();
				server.log("ConnectionManager: Accepting new connection.");
			} catch (IOException e) {
				server.log("ConnectionManager: Server socket is closed.");
			}
		}
	}
	
	/**
	 * Closes this connection manager and all connections.
	 */
	public void close() {
		
		server.log("ConnectionManager: Closing all connections.");
		
		running = false;
		
		try {
			Enumeration<XMLConnection> enumer = authConnections.elements();
			
			listener.close();
			
			//TODO: sende close melding til klientene??
			
			/* Close authenticated connections */
			while (enumer.hasMoreElements()) {
				enumer.nextElement().close();
			}
			
			/*Close unauthenticated connections */
			for (Connection c: connections) {
				c.close();
			}
		} catch (IOException e) {
			server.log("ConnectionManager: Problem closing server socket.");
		}
	}
	
	/**
	 * Removes a given connection from the connection list.
	 * 
	 * @param c The connection to be closed.
	 */
	public void closeConnection(Connection c) {		
		if (c.isAuthenticated()) {
			authConnections.remove(c.getUsername());
			server.removeAuthedConnection(c);
		} else {
			connections.remove(c);
			server.removeNonAuthedConnection(c);
		}
	}
	
	/**
	 * Authenticates a given connection with an assosiated username.
	 * 
	 * @param username The username assosiated with the connection.
	 * @param con The connection to be authenticated.
	 */
	public boolean autenticateConnection(String username, String version, XMLConnection con) {
		server.log("ConnectionManager: Authenticated connection: " + username);
		
		/* If there already exist an authed connection for this user, the old
		 * connection is replaced with the new one. A user can only have one
		 * running XMLConnection */
		if(authConnections.containsKey(username)) {
			server.log("authConnections inneholdt allerede en connection emd dette brukernavnet");
			XMLConnection old = authConnections.remove(username);
			server.removeAuthedConnection(old);
			old.close();
			//TODO: Evt sende melding til old om at den lukkes fr vi lukker den
		}
		
		authConnections.put(username, con);
		connections.remove(con);
		server.removeNonAuthedConnection(con);
		server.addAuthedConnection(con);
		con.autenticate(username);
		con.setVersion(version);
		
		//TODO: sjekk p om det gikk
		return true;
	}
	
	/**
	 * Method for retreiving the InetAddress of a connection from the 
	 * assosiated username.
	 * 
	 * @param username The username assosiated with the connection,
	 * @return the InetAddress of the connection with the assosiated username.
	 */
	public InetAddress getClientInetAddress(String username) {
		Connection c = authConnections.get(username);
		if (c != null) {
			return c.getInetAddress();
		} else {
			return null;
		}
	}
	
	/**
	 * Returns the authenticated connection with the assosiated username or null
	 * if no such connection exists.
	 * 
	 * @param username The username assosiated with the authenticated connection.
	 * @return The connection assosiated with the given username.
	 */
	public XMLConnection getClientConnection(String username) {
		return authConnections.get(username);
		
	}
	
	/**
	 * Gets the hashtable containing all authenticated connections.
	 * 
	 * @return the hashtable containing all authenticated connections.
	 */
	public Hashtable<String, XMLConnection> getAuthedConnections() {
		return authConnections;
	}
}
