package digital_home;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.ArrayList;

import org.jdom.Document;
import org.jdom.Element;

/**
 * A class representing a connection used for transfering files 
 * from the server to a client. 
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class DownloadConnection extends Connection {

	/* Buffer variables */
	private static final int BUFFER_SIZE = 1024;
	private byte[] buffer;
	
	private ArrayList<Element> queue;
	private FileTransferManager ftMan;
	private XMLConnection xmlCon;
	
	/**
	 * Creates a new download connection.
	 * 
	 * @param server The server this connection belongs to.
	 * @param socket The socket used for the file transfer.
	 * @param username The username of the client using this connection.
	 */
	public DownloadConnection(Server server, Socket socket, String username) {
		super(server, socket);
		
		this.ftMan = server.getFileTransferManager();
		super.username = username;
		queue = new ArrayList<Element>();
		buffer = new byte[BUFFER_SIZE];
		xmlCon = server.getConnectionManager().getClientConnection(username);
	}
	
	/**
	 * Adds a download request to the download queue.
	 * 
	 * @param request The download request.
	 */
	public void addRequest(Element request) {
		queue.add(request);
	}
	
	public void setQueue(ArrayList<Element> queue) {
		this.queue = queue;
	}
	
	/**
	 * Sends the file information to the client.
	 * 
	 * @param transfer The transfer from which information is to be sent.
	 */
	private void sendFileInfo(Element transfer) {
		String fileType = transfer.getChildText(XMLVariables.FILE_TYPE);
		String fileName = transfer.getChildText(XMLVariables.FILE_NAME);
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Element body = new Element(XMLVariables.SERVICE_BODY);
		Element fileTypeElement = new Element(XMLVariables.FILE_TYPE);
		Element fileNameElement = new Element(XMLVariables.FILE_NAME);
		Document doc = new Document(root);
		
		root.setAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_DOWNLOAD_ID);
		fileTypeElement.setText(fileType);
		fileNameElement.setText(fileName);
		
		body.addContent(fileTypeElement);
		body.addContent(fileNameElement);
		root.addContent(body);
		
		xmlCon.sendXMLDoc(doc);
				
		server.log("DownloadConnection - " + username + 
				": Sending file info to client: " + fileName + "." + fileType);
	}
	
	
	/**
	 * Sends first an XML document containing file info, then sends the requested file.
	 * After finished sending, calls this method with the next request in the queue if any.
	 * 
	 * @param transfer The transfer to be processed.
	 */
	private void sendFile(Element transfer) {		
		String fileType = transfer.getChildText(XMLVariables.FILE_TYPE);
		String filePath = transfer.getChildText(XMLVariables.FILE_PATH);
		String fileName = transfer.getChildText(XMLVariables.FILE_NAME);
		
		server.log("DownloadConnection - " + username + 
				": Sending file to client: " + fileName + "." + fileType);
		
		try {
			BufferedInputStream bufferedIn = new BufferedInputStream(new FileInputStream(filePath+fileName+ "." + fileType));
			BufferedOutputStream bufferedOut = new BufferedOutputStream(socket.getOutputStream());
			int len = 0;

			/* Sending actual file */
			while ((len = bufferedIn.read(buffer)) > 0) {
				bufferedOut.write(buffer, 0, len);
			}
			
			bufferedIn.close();
			bufferedOut.flush(); 
			
		} catch (IOException e) {
			server.log("DownloadConnection - " + username + 
					": Error occured when sending file to client.");
		}
	}
	
	/**
	 * Sends a close message to the client.
	 *
	 */
	private void sendClose() {
		server.log("DownloadConnection - " + username + 
				": Sending close message to the client.");
		
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Element body = new Element(XMLVariables.SERVICE_BODY);
		Element status = new Element(XMLVariables.STATUS);
		Document doc = new Document(root);
		
		root.setAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_DOWNLOAD_ID);
		status.setText(XMLVariables.STATUS_CLOSE);
		body.addContent(status);
		root.addContent(body);
		
		xmlCon.sendXMLDoc(doc);
		
		close();
	}

	/**
	 * Method called when starting the connection thread.
	 * Initialized the sending of the first element in the download queue.
	 */
	public void run() {	
		sendFile(queue.get(0));
	}
	
	/**
	 * Starts the file transfer process by sending file info to the client.
	 *
	 */
	public void startTransfer() {
		sendFileInfo(queue.get(0));
	}
	
	/**
	 * Starts the actual file transfer. This method is called when the server
	 * receives file info received message.
	 *
	 */
	public void activateFileTransfer(){
		Thread t = new Thread(this);
		
		t.start();
	}
	
	/**
	 * Method called when file ok message is received. This method 
	 * starts a new file transfer process if there are any files in queue.
	 * If the queue is empty, this method sends a close method to the client.
	 */
	public void fileOkRecieved()
	{
		/* Removes from the queue here to be able to resend if the transmition failed */
		queue.remove(0);
		
		if (!queue.isEmpty()) {
			sendFileInfo(queue.get(0));
		} else {
			sendClose();
		}
	}
		
	/**
	 * Closes this connection. 
	 */
	public void close() {
		
		server.log("DownloadConnection - " + username +
				": Closing connection.");
		
		try {
			out.close();
			in.close();
			socket.close();
			ftMan.closeConnection(this);
		} catch (IOException e) {
			server.log("DownloadConnection - " + username + 
					": Failed to close connection.");
		}
	}

}
