package digital_home;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Enumeration;
import java.util.Hashtable;

import org.jdom.Document;
import org.jdom.Element;

/**
 * A class managing all downloads and uploads.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class FileTransferManager {
	
	private Server server;
	
	private Hashtable<String, DownloadConnection> downloadConnections;
	private Hashtable<String, UploadConnection> uploadConnections;
	private Hashtable<String, UploadWaitController> uploadWaitControllers;
	
	/**
	 * Creates a new File Transfer Manager.
	 * 
	 * @param server The server this manager belongs to.
	 */
	public FileTransferManager(Server server) {
		this.server = server;
		downloadConnections = new Hashtable<String, DownloadConnection>();
		uploadConnections = new Hashtable<String, UploadConnection>();
		uploadWaitControllers = new Hashtable<String, UploadWaitController>();
	}
	
	/**
	 * Receives a download request and starts a new connection if needed and 
	 * queues the download request. The connection, if initialized, is started as a new thread. 
	 * 
	 * @param username The user who requested the download.
	 * @param request The download request.
	 */
	public void receiveDownloadRequest(String username, Element request) {
		//TODO: hndtering av hengende downloadconnections
		// f.eks. klient som har krasjet n server bare har sendt filinfo
		
		server.log("FileTransferManager: Received download request from " + username);
		
		DownloadConnection downloadCon = null;	
	
		if (downloadConnections.get(username) != null){		
			/* If the connection already exists, the new request is added to the queue */
			downloadCon = downloadConnections.get(username);
			downloadCon.addRequest(request);
		} else {
			/* Else a new connection is made, then the reques is added to the queue */
			downloadCon = createNewDownloadConnection(username);
		
			/* Starting new connection thread */
			downloadCon.addRequest(request);
			downloadCon.startTransfer();	
		}
	}
	
	/**
	 * Creates a new download connection for the user with the given username.
	 * 
	 * @param username the username for which the download connection is going to be created.
	 * @return the new download connection.
	 */
	public DownloadConnection createNewDownloadConnection(String username) {
		DownloadConnection downloadCon = null;
		
		try {
			Socket socket;
			InetAddress ip = server.getConnectionManager().getClientInetAddress(username);
			
			if (ip != null) {
				/* Creating connection to client */
				socket = new Socket(ip, NetworkVariables.DOWNLOAD_PORT);
				downloadCon = new DownloadConnection(server, socket, username);
				downloadConnections.put(username, downloadCon);
			
				server.addDownloadConnection(downloadCon);
				server.log("FileTransferManager: Creating new download " +
					"connection with user " + username);
			}
		} catch (IOException e) {
			server.log("FileTransferManager: Failed to create new download " +
					"connection with user " + username);
		}
		return downloadCon;
	}
	
	/**
	 * Creates a new upload connection for the user with the given username.
	 * 
	 * @param userName the username for which the upload connection is going to be created.
	 * @return the new download connection.
	 */
	public UploadConnection createNewUploadConnection(String userName) {
		UploadConnection uploadCon = null;
		
		try {
			Socket socket;
			InetAddress ip = server.getConnectionManager().getClientInetAddress(userName);
			
			if (ip != null) {
				/* Creating connection to client */
				socket = new Socket(ip, NetworkVariables.UPLOAD_PORT);
				uploadCon = new UploadConnection(server, userName, socket);
				uploadConnections.put(userName, uploadCon);
			
				server.log("FileTransferManager: Creating new upload " +
					"connection with user " + userName);
			}
		} catch (IOException e) {
			server.log("FileTransferManager: Failed to create new upload " +
					"connection with user " + userName);
		}
		return uploadCon;
	}
		
	
	/**
	 * Method called when file info is received from a user.
	 * 
	 * @param username The username of the connection on which to send file info.
	 */
	public void fileInfoReceived(String username) {
		DownloadConnection downloadCon = downloadConnections.get(username);;	
		
		if (downloadCon != null){		
			downloadCon.activateFileTransfer();
		} else {
			server.log("FileTransferManager: Can't send file info because" +
					" connection does not exist.");
		}
	}
	
	/**
	 * Method called when file ok message is received.
	 * 
	 * @param username The username of the connection from which the message was from.
	 */
	public void fileOkReceived(String username) {
		DownloadConnection downloadCon = downloadConnections.get(username);;	
		
		if (downloadCon != null){		
			downloadCon.fileOkRecieved();
		} else {
			server.log("FileTransferManager: File OK received, but connection was lost.");
		}
	}
		
	/**
	 * Receives a upload request and starts a new connection if needed and 
	 * queues the upload request. The connection, if initialized, is started as a new thread. 
	 * 
	 * @param username The user who requested the upload.
	 * @param request The upload request.
	 */
	public String receiveUploadRequest(String username, Element request, XMLConnection xmlCon) {
		
		/* Checks if the file already is uploaded to the server */
		String completeFileName = request.getChildText(XMLVariables.FILE_NAME) + "." 
								  + request.getChildText(XMLVariables.FILE_TYPE);

		File uploadedFile = new File(NetworkVariables.UPLOAD_URL + username + "\\" 
									 + completeFileName);
		
		if (uploadedFile.exists()) {
			return XMLVariables.RMI_STATUS_SUCCESS;
		}
		
		/* Creating upload connection if necessary */
		UploadConnection uploadCon = uploadConnections.get(username);
		boolean uploadConnectionExists = true;
		
		if (uploadCon == null) {
			uploadCon = createNewUploadConnection(username);
			uploadConnectionExists = false;
			
			Thread t = new Thread(uploadCon);
			t.start();
		}
		
		/* Sending upload request to the client */
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Document fileRequest = new Document(root);
		
		root.setAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_UPLOAD_ID);
		root.addContent((Element) request.clone());
		
		xmlCon.sendXMLDoc(fileRequest);
		
		/* Giving response to user */ 
		if (uploadConnectionExists) {
			return XMLVariables.RMI_STATUS_QUEUED;
		} else {
			/* Creating upload waiter class */ 
			UploadWaitController uploadWait = new UploadWaitController();
			uploadWaitControllers.put(username, uploadWait);
			
			/* Start to wait */
			uploadWait.setWaiting(true);
			
			/* Returns aftern waiting */
			if (uploadCon.isNoFile()) {
				uploadCon.setNoFile(false);
				return XMLVariables.RMI_STATUS_FAILED;
			} else {
				return XMLVariables.RMI_STATUS_SUCCESS;				
			}		
		}
	}
	
	/**
	 * Removes a closed connection from the right connection list.
	 * 
	 * @param con The closed connection.
	 */
	public void closeConnection(Connection con) {
		if (con instanceof DownloadConnection) {
			downloadConnections.remove(con.getUsername());
			server.removeDownloadConnection(con);
		} else if (con instanceof UploadConnection) {
			uploadConnections.remove(con.getUsername());
		}
	}
	
	/**
	 * Closes a download connection if available.
	 * 
	 * @param username The username of the download connection.
	 */
	public void closeDownloadConnection(String username) {
		DownloadConnection con = downloadConnections.get(username);
		if (con != null) {
			con.close();
		}
	}
	
	/**
	 * Closes all connections.
	 */
	public void close() {
		server.log("FileTransferManager: Closing all download connections.");
		
		Enumeration<DownloadConnection> enumerD = downloadConnections.elements();
		
		while (enumerD.hasMoreElements()) {
			DownloadConnection dc = enumerD.nextElement();
			//TODO: sende melding til klient?
			dc.close();
		}
		
		Enumeration<UploadConnection> enumerU = uploadConnections.elements();
		
		while (enumerU.hasMoreElements()) {
			UploadConnection uc = enumerU.nextElement();

			closeUploadConnection(uc.getUsername());
		}
	}
	
	/**
	 * Creates a file upload status XML document with the chosen status.
	 * 
	 * @param status the status of the upload status document.
	 * @return the created XML document.
	 */
	private Document createUploadStatusMessage(String status) {
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Element body = new Element(XMLVariables.SERVICE_BODY);
		Element statusElement = new Element(XMLVariables.STATUS);
		Document doc = new Document(root);
		
		statusElement.setText(status);
		root.setAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_UPLOAD_ID);
		body.addContent(statusElement);
		root.addContent(body);
		
		return doc;
	}

	/**
	 * Creates and sends an upload file status message stating that the
	 * uploaded file is received.
	 * 
	 * @param username the username of the connection.
	 * @param xmlCon the XMLConnection through which to send the message.
	 */
	public void sendUploadFileOk(String username, XMLConnection xmlCon) {
		/* Sending file ok message to client */
		Document doc = createUploadStatusMessage(XMLVariables.STATUS_FILE_OK);
		
		xmlCon.sendXMLDoc(doc);
		
		/* Waking up sleeping thread to give user a response */	
		UploadWaitController uploadWait = uploadWaitControllers.remove(username);
		if (uploadWait != null) {
			uploadWait.setWaiting(false);
		}
	}
	
	/**
	 * Message called from parser when file information about a file that is 
	 * going to be uploaded is received.
	 * 
	 * @param username the username from which the file is being uploaded from.
	 * @param fileName the name of the file.
	 * @param fileType the extension of the file.
	 * @param xmlCon the XMLConnection the message came from.
	 */
	public void receivedUploadFileInfo(String username, String fileName, String fileType, XMLConnection xmlCon) {
		/* Sets the complete file path at the upload connection, prepares to receive file */
		UploadConnection uploadCon = uploadConnections.get(username);
		String completePath = NetworkVariables.UPLOAD_URL + username + "\\" + fileName + "." + fileType;
		File userDir = new File(NetworkVariables.UPLOAD_URL + username);
		
		/* If the destination dir does not exist, create it */
		if (!userDir.exists()) {
			userDir.mkdir();
		}
		
		uploadCon.setCompleteFilePath(completePath);
		
		/* Informs the client that the server is ready to receive file */
		Document doc = createUploadStatusMessage(XMLVariables.STATUS_FILEINFO_OK);
		xmlCon.sendXMLDoc(doc);
	}
	
	/**
	 * Message called from parser if the requested file for upload was
	 * unavailable at the client.
	 * 
	 * @param username the username from which the file is being uploaded from.
	 * @param xmlCon the XMLConnection the message came from.
	 */
	public void setNoUploadFileReceived(String username, XMLConnection xmlCon) {
		
		/* Checks if a user is waiting for a reply */
		UploadWaitController uploadWait = uploadWaitControllers.get(username);
		
		if (uploadWait != null) {
			UploadConnection uploadCon = uploadConnections.get(username);
			uploadCon.setNoFile(true);
			
			uploadWait.setWaiting(false);
			uploadWaitControllers.remove(username);
		}
		
		/* Tells the client to prepare next file for transfer */
		sendUploadFileOk(username, xmlCon);
	}
	
	/**
	 * Method that closes an upload connection with the given username.
	 * 
	 * @param username the username of the connection to be closed.
	 */
	public void closeUploadConnection(String username) {
		UploadConnection uploadCon = uploadConnections.get(username);
		UploadWaitController uploadWait = uploadWaitControllers.remove(username);
		
		if (uploadWait != null) {
			uploadCon.setNoFile(true);
			uploadWait.setWaiting(false);
		}
		
		uploadCon.close();
	}
}
