package digital_home;

import java.io.BufferedReader;
import java.io.FileReader;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.jdom.Document;
import org.jdom.Element;

import digital_home.db.*;
import digital_home.parser.*;
import digital_home.rmi.*;

/**
 * The main server class
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class Server {
	
	private final String GAC_FILE_NAME = "gacutil";
	private final String GAC_FILE_TYPE = "exe";
	private final String GAC_FILE_PATH = "update\\";
	private final String UPDATE_FILE_NAME = "DigitalHomeClient";
	private final String UPDATE_FILE_TYPE = "dll";
	private final String UPDATE_FILE_PATH = "update\\";
	
	private ConnectionManager conMan;
	private ParseManager parseMan;
	private FileTransferManager ftMan; 
	
	private RMIReceiver rmiReceiver;
	
	private ServerGUI gui;
	
	/* Not used in the prototype */
	private UserDatabase db; 
	
	private String latestClientVersion;
	
	/**
	 * Constructor creating a new server instance
	 */
	public Server() {
		ftMan = new FileTransferManager(this);
		parseMan = new ParseManager(this);
		conMan = new ConnectionManager(this);
		db = new UserDatabase();
		
		latestClientVersion = readLatestClientVersion();
				
		/* Start connection manager */
		Thread t = new Thread(conMan);
		t.start();
		
		/* Register RMI process */
		try {
			rmiReceiver = new RMIReceiver(this);
			System.setSecurityManager(new LiberalSecurityManager());
			Registry registry = LocateRegistry.createRegistry(NetworkVariables.RMI_PORT);
			Naming.rebind(NetworkVariables.RMI_URL, rmiReceiver);
		} catch (RemoteException e) {
			// TODO fikse feilhndtering
			e.printStackTrace();
		} catch (MalformedURLException e) {
			// TODO fikse feilhndtering
			e.printStackTrace();
		}
		
		gui = new ServerGUI(this);
	}
	
	/**
	 * Reads the latest client version from the version file.
	 * 
	 * @return the latest client version.
	 */
	private String readLatestClientVersion() {
		try {
			BufferedReader reader = new BufferedReader(new FileReader(UPDATE_FILE_PATH + "version.txt"));
			return reader.readLine();
		} catch (Exception e) {
			log("Server: Couldn't read client latest version.");
			return null;
		}
	}
	
	/**
	 * Method used when the server receives an XML document. The document is passed on to a parse manager 
	 * 
	 * @param doc	XML document to be parsed
	 * @param con	Connection used to send XML replies to the client Media Center PC
	 */
	public Document receiveXMLDoc(Document doc, XMLConnection con) {	
		XMLConnection con2 = con;
		log("Server: XML document received.");
		
		if (con == null) {
			Element root = doc.getRootElement();
			String username;
			
			/* Error handling. The document does not contain a root element */
			if (root == null) {
				//TODO: fikse feilmelding
				log("Server: XML document received - Wrong format.");
				return createMCEConnectionResponse(false);
			}
			
			username = root.getAttributeValue(XMLVariables.SERVICE_USER);
			
			/* Error handling. No user spesified */
			if (username == null) { 
				//TODO: fikse feilmelding
				log("Server: XML document received - Does not contain username.");
				return createMCEConnectionResponse(false);
			}
			
			con2 = conMan.getClientConnection(username);
			
			/* Error handling. The user Media Center PC is not connected to the server */
			if (con2 == null) {
				log("Server: XML document received - No matching connection found.");
				return createMCEConnectionResponse(false);
			}
		}
		
		return parseMan.distributeXMLDoc(doc, con2);
	}
	
	/**
	 * Creates a server system response informing whether or not the MCE
	 * of the user is connected.
	 * 
	 * @param status true if the MCE was connected, else false.
	 * @return the server system response document.
	 */
	public Document createMCEConnectionResponse(boolean status) {
		Element root = new Element(XMLVariables.SERVER_SYSTEM_RESPONSE);
		Document response = new Document(root);
		
		root.setAttribute(XMLVariables.SERVER_SYSTEM_MCE_CONNECTED, Boolean.toString(status));
		
		return response;
	}
	
	/**
	 * Creating a server system response with status error.
	 * 
	 * @return the server system response document.
	 */
	public Document createErrorResponse() {
		Element root = new Element(XMLVariables.SERVER_SYSTEM_RESPONSE);
		Document response = new Document(root);
		
		root.setAttribute(XMLVariables.STATUS, XMLVariables.STATUS_ERROR);
		
		return response;
	}
	
	/**
	 * Updates the client software with the given connection.
	 * 
	 * @param con the XMLConnection of the client.
	 */
	public void sendUpdatedClientSoftware(XMLConnection con) {
		Document gacRequest = createFileTransferRequest(GAC_FILE_NAME, GAC_FILE_TYPE, GAC_FILE_PATH);
		Document updateRequest = createFileTransferRequest(UPDATE_FILE_NAME, UPDATE_FILE_TYPE, UPDATE_FILE_PATH);
		
		this.receiveXMLDoc(gacRequest, con);
		this.receiveXMLDoc(updateRequest, con);
	}
	
	/**
	 * Creates and sends a download request for both the GAC modifier and the update file
	 * to all online connections.
	 */
	public void sendUpdatedClientSoftware() {
		Hashtable<String, XMLConnection> authedConnections = conMan.getAuthedConnections();
		Enumeration<XMLConnection> authedConnectionsEnum = authedConnections.elements();
		
		latestClientVersion = readLatestClientVersion();
				
		while (authedConnectionsEnum.hasMoreElements()) {
			XMLConnection con = authedConnectionsEnum.nextElement();
			sendUpdatedClientSoftware(con);
		}
	}
	
	/**
	 * Creates a file transfer request document.
	 * 
	 * @param fileName The file name of the file.
	 * @param fileType The file type of the file.
	 * @param filePath The file path of the file.
	 * @return The finished file transfer request document.
	 */
	private Document createFileTransferRequest(String fileName, String fileType, String filePath) {
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Element body = new Element(XMLVariables.SERVICE_BODY);
		Element fileNameElement = new Element(XMLVariables.FILE_NAME);
		Element fileTypeElement = new Element(XMLVariables.FILE_TYPE);
		Element filePathElement = new Element(XMLVariables.FILE_PATH);
		Document doc = new Document(root);
		
		fileNameElement.setText(fileName);
		fileTypeElement.setText(fileType);
		filePathElement.setText(filePath);
		
		body.addContent(fileNameElement);
		body.addContent(fileTypeElement);
		body.addContent(filePathElement);
		
		root.addContent(body);
		root.setAttribute(XMLVariables.SERVICE_ID, XMLVariables.FILE_DOWNLOAD_ID);
		
		return doc;
	}
	
	/**
	 * Checks whether or not a new client version of the software exists.
	 * 
	 * @param version the client version.
	 * @return true if a new client version exists, else false.
	 */
	public boolean newClientVersionExist(String version){
		StringTokenizer st = new StringTokenizer(version, ".");
		int c1 = Integer.parseInt(st.nextToken());
		int c2 = Integer.parseInt(st.nextToken());
		
		st = new StringTokenizer(latestClientVersion, ".");
		int s1 = Integer.parseInt(st.nextToken());
		int s2 = Integer.parseInt(st.nextToken());

		if ((s1 > c1) || (s1 == c1 && s2 > c2)) {
			return true;
		} else {
			return false;	
		}
	}
	
	/**
	 * Sends a message which is going to be displayed in a pop-up box in the 
	 * client's MCE.
	 * 
	 * @param title the title of the message.
	 * @param message the message.
	 * @param username the username of the client.
	 * @return true if the client was connected, else false.
	 */
	public boolean sendMessageToClient(String title, String message, String username) {
		XMLConnection con = conMan.getClientConnection(username);
		
		if (con != null) {
			sendMessageToClient(title, message, con);
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Sends a message which is going to be displayed in a pop-up box in the 
	 * client's MCE.
	 * 
	 * @param title the title of the message.
	 * @param message the message.
	 * @param con the XMLConnection through which to send the message.
	 */
	public void sendMessageToClient(String title, String message, XMLConnection con) {
		Element root = new Element(XMLVariables.SERVICE_ROOT);
		Element body = new Element(XMLVariables.SERVER_SYSTEM_RESPONSE);
		Element titleElement = new Element("title");
		Element messageElement = new Element("message");
		Document doc = new Document(root);
		
		titleElement.setText(title);
		messageElement.setText(message);
		body.addContent(titleElement);
		body.addContent(messageElement);
		root.addContent(body);
		root.setAttribute(XMLVariables.SERVICE_ID, "MessageToUser");
		
		con.sendXMLDoc(doc);
	}
	
	/**
	 * Method used to get the server's connection manager
	 * 
	 * @return	The servers connection manager
	 */
	public ConnectionManager getConnectionManager() {
		return conMan;
	}
	
	/**
	 * Method used to get the server's file transfer manager
	 * 
	 * @return	The servers file transfer manager
	 */
	public FileTransferManager getFileTransferManager() {
		return ftMan;
	}
	
	/**
	 * Prints a String in the log box of the GUI.
	 * 
	 * @param entry The String to be printede.
	 */
	public void log(String entry) {
		gui.log(entry);
	}
	
	/**
	 * Adds a non-authed connection to the GUI list.
	 * 
	 * @param con The connection to be added.
	 */
	public void addNonAuthedConnection(Connection con) {
		gui.addNonAuthedConnection(con);
	}
	
	/**
	 * Removes a non-authed connection to the GUI list.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeNonAuthedConnection(Connection con) {
		gui.removeNonAuthedConnection(con);
	}
	
	/**
	 * Adds an authed connection to the GUI list.
	 * 
	 * @param con The connection to be added.
	 */
	public void addAuthedConnection(Connection con) {
		gui.addAuthedConnection(con);
	}
	
	/**
	 * Removes an authed connection to the GUI list.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeAuthedConnection(Connection con) {
		gui.removeAuthedConnection(con);
	}
	
	/**
	 * Adds a download connection to the GUI list.
	 * 
	 * @param con The connection to be added.
	 */
	public void addDownloadConnection(Connection con) {
		gui.addDownloadConnection(con);
	}
	
	/**
	 * Removes a download connection to the GUI list.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeDownloadConnection(Connection con) {
		gui.removeDownloadConnection(con);
	}
	
	/**
	 * Main method
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.print("Initializing server....");
		new Server();
		System.out.println("....server started!");
	}
}
