package digital_home;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Class representing a simple GUI used by the server. The GUI shows
 * a list of all non-authed, authed and download connections as well
 * as a text area containing the server log.
 * 
 * @author Andreas W. Bjerkhaug.
 * @author ystein Ellingb.
 */
public class ServerGUI extends JFrame implements ActionListener {
	
	/* The server this GUI belongs to */
	private Server server;
	
	/* List models used to represent the content of the JLists */
	private DefaultListModel nonAuthedModel;
	private DefaultListModel authedModel;
	private DefaultListModel downloadModel;
	
	/* Buttons and menu items */
	private JButton close;
	private JMenuItem exit;
	private JMenuItem update;
	private JMenuItem about;
	
	/* The text area used for server log */
	private JTextArea logArea;
	
	/**
	 * Creates the GUI.
	 * 
	 * @param server The server this GUI belongs to.
	 */
	public ServerGUI(Server server) {
		super("Digital Home Server");
		
		this.server = server;
		
		/* Building the GUI window */
		try{
		     UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		 }
		 catch(Exception e){}

		 this.addWindowListener(new WindowAdapter(){

			 public void windowClosing(WindowEvent e) {
				 exit();
			 }

		     public void windowClosed(WindowEvent e) {}
		 });
		 
		 setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE ); 
		 setSize(640,480);
		 setResizable(false);
		 addComponents();
		 setLocationRelativeTo(null);
		 setVisible(true);
	}
	
	/**
	 * Method creating the GUI.
	 */
	private void addComponents() {
		JPanel mainPanel = new JPanel(new BorderLayout());
		JPanel labelPanel = new JPanel(new GridLayout(1,3));
		JPanel listPanel = new JPanel(new GridLayout(1,3));
		JPanel midPanel = new JPanel(new GridLayout(2,1));
		JPanel buttonPanel = new JPanel();
		JScrollPane nonAuthedScroll, authedScroll, downloadScroll;
		JLabel nonAuthedLabel, authedLabel, downloadLabel;
		JMenuBar menuBar = new JMenuBar();
		JMenu file, help, server;
		
		/* Creating the label panel */
		nonAuthedLabel = new JLabel(" Non-authed connections:");
		authedLabel = new JLabel(" Authed connections:");
		downloadLabel = new JLabel(" Download connections:");
		
		labelPanel.add(nonAuthedLabel);
		labelPanel.add(authedLabel);
		labelPanel.add(downloadLabel);
		
		/* Creating the list panel */
		nonAuthedModel = new DefaultListModel();
		JList nonAuthedConnections = new JList(nonAuthedModel);
		nonAuthedScroll = new JScrollPane(nonAuthedConnections, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
										  JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listPanel.add(nonAuthedScroll);
		
		authedModel = new DefaultListModel();
		JList authedConnections = new JList(authedModel);
		authedScroll = new JScrollPane(authedConnections, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
									   JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listPanel.add(authedScroll);
		
		downloadModel = new DefaultListModel();
		JList downloadConnections = new JList(downloadModel);
		downloadScroll = new JScrollPane(downloadConnections, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
										 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listPanel.add(downloadScroll);
		
		/* Creating the button panel */
		close = new JButton("Close");
		close.addActionListener(this);
		buttonPanel.setLayout(new GridLayout(1,3));
		buttonPanel.add(new JLabel(""));
		buttonPanel.add(close);
		buttonPanel.add(new JLabel(""));
		
		/* Creating the menu bar */
		exit = new JMenuItem("Exit");
		exit.addActionListener(this);
		
		update = new JMenuItem("Update client software");
		update.addActionListener(this);
		
		about = new JMenuItem("About");
		about.addActionListener(this);
		
		file = new JMenu("File");
		file.add(exit);
		
		server = new JMenu("Server");
		server.add(update);
		
		help = new JMenu("Help");
		help.add(about);
		
		menuBar.add(file);
		menuBar.add(server);
		menuBar.add(help);
		this.setJMenuBar(menuBar);
		
		/* Creating the log area */
		logArea = new JTextArea();
		logArea.setEditable(false);
		JScrollPane logPane = new JScrollPane(logArea, 
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		
		/* Creating the main panel */
		midPanel.add(listPanel);
		midPanel.add(logPane);
		mainPanel.add(labelPanel, BorderLayout.NORTH);
		mainPanel.add(midPanel, BorderLayout.CENTER);
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		this.getContentPane().add(mainPanel);
	}
	
	/**
	 * Adds a connection to the list of non-authed connections.
	 * 
	 * @param con The connection to be added.
	 */
	public void addNonAuthedConnection(Connection con) {
		nonAuthedModel.addElement(con);
	}
	
	/**
	 * Removes a connection from the list of non-authed connections.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeNonAuthedConnection(Connection con) {
		nonAuthedModel.removeElement(con);
	}
	
	/**
	 * Adds an connection to the list of authed connections.
	 * 
	 * @param con The connection to be added.
	 */
	public void addAuthedConnection(Connection con) {
		authedModel.addElement(con);
	}
	
	/**
	 * Removes a connection from the list of authed connections.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeAuthedConnection(Connection con) {
		authedModel.removeElement(con);
	}
	
	/**
	 * Adds a connection to the list of non-authed connections.
	 * 
	 * @param con The connection to be added.
	 */
	public void addDownloadConnection(Connection con) {
		downloadModel.addElement(con);
	}
	
	/**
	 * Removes a connection from the list of download connections.
	 * 
	 * @param con The connection to be removed.
	 */
	public void removeDownloadConnection(Connection con) {
		downloadModel.removeElement(con);
	}
	
	/**
	 * Adds the log entry to the log.
	 * 
	 * @param logEntry The log entry to be added.
	 */
	public void log(String logEntry) {
		logArea.append(logEntry + "\n");
		logArea.setCaretPosition(logArea.getDocument().getLength());
	}
	
	/**
	 * Method creating a confirm box and exits the program if the user
	 * selects that choice.
	 */
	private void exit() {
		int choice = JOptionPane.showConfirmDialog(ServerGUI.this,
				 "Are you sure you want to exit?",
				 "Exit?",
				 JOptionPane.YES_NO_OPTION);
		
		if (choice == JOptionPane.YES_OPTION){
			server.getConnectionManager().close();
			server.getFileTransferManager().close();
			System.exit(0);
		} 
	}

	/**
	 * Method defining which action to perform when the user push different buttons.
	 */
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == close || event.getSource() == exit) {
			exit();
		} else if (event.getSource() == about) {
			JOptionPane.showMessageDialog(this,
				      "Digital Home - A collaboration between Accenture and NTNU \n\n" +
				      "Andreas W. Bjerkhaug\n" +
				      "ystein Ellingb\n\n" +
				      "Copyright (C) 2006.",
				      "About...",
				      JOptionPane.PLAIN_MESSAGE);

		} else if (event.getSource() == update) {
			server.sendUpdatedClientSoftware();
			JOptionPane.showMessageDialog(this,
				      "Updated client software sent to all online clients.",
				      "Updates sent",
				      JOptionPane.INFORMATION_MESSAGE);
		}
	}
}
