package digital_home;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.ArrayList;

import org.jdom.Element;

/**
 * A class representing a connection used for transfering files 
 * from a client to the server. 
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class UploadConnection extends Connection {

	/* Buffer variables */
	private static final int BUFFER_SIZE = 1024;
	private byte[] buffer;
	
	private String completeFilePath;
	private boolean isFinishedTransmitting;
	
	private boolean noFile;
		
	private XMLConnection xmlCon;
		
	/**
	 * Creates a new upload connection for a given user.
	 * 
	 * @param server the server this connection belongs to.
	 * @param username the username of the client this connection connects to.
	 * @param socket the socket through which to communicate.
	 */
	public UploadConnection(Server server, String username, Socket socket) {
		super(server, socket);
		
		buffer = new byte[BUFFER_SIZE];

		completeFilePath = null;
		isFinishedTransmitting = false;
		
		noFile = false;
		
		super.username = username;
		
		xmlCon = server.getConnectionManager().getClientConnection(username);
	}
	
	/**
	 * Method called when file information is received from client.
	 * This message makes the upload connection wait for incoming file.
	 * 
	 * @param completeFilePath
	 */
	public synchronized void setCompleteFilePath(String completeFilePath) {
		server.log("UploadConnection: Awakened from sleep.");
		
		this.completeFilePath = completeFilePath;
		notify();
	}
	
	/**
	 * Message telling whether the noFile flag has been set.
	 * 
	 * @return true if the noFile flag has been set.
	 */
	public boolean isNoFile() {
		return noFile;
	}
	
	/**
	 * Sets the noFile flag. This is going to be set to true
	 * if the client did not have the expected file.
	 * 
	 * @param noFile true if the client did not have the expected file. 
	 */
	public void setNoFile(boolean noFile) {
		this.noFile = noFile;
	}

	/**
	 * Method for receiving files.
	 */
	public synchronized void run() {
		
		while (running) {
			try {
				
				try {
					wait();
					server.log("UploadConnection: Successfully awakened.");
				} catch (InterruptedException e) {
				}
				
				if (isFinishedTransmitting) {
					break;
				}
				
				File file = new File(completeFilePath);
				BufferedInputStream bufferedIn = new BufferedInputStream(in);
				BufferedOutputStream bufferedOut = new BufferedOutputStream(new FileOutputStream(file));	

				int len = 0;

				/* Receiving actual file */
				
				server.log("UploadConnection: Waiting to receive file.");
				//TODO: fikse timeout, available()
				int waitCounter = 0;
				
				while (true) {
										
					if (bufferedIn.available() > 0) {
						if ((len = bufferedIn.read(buffer)) > 0) {
							bufferedOut.write(buffer, 0, len);
							waitCounter = 0;
						}
					} else {
						if (waitCounter > 10) {
							break;
						}
			
						try {
							Thread.sleep(10);
							waitCounter++;
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}				
				}
				/* Flushes and closes the streams and the file */
				bufferedIn.close();
				bufferedOut.flush(); 
				bufferedOut.close();
				
				server.log("UploadConnection: Finished receiving file.");
				
				/* Sends a "file OK" message to the client */
				FileInfoTokenizer fit = new FileInfoTokenizer(completeFilePath);
				
				server.getFileTransferManager().sendUploadFileOk(username, xmlCon);

				completeFilePath = null;
				
			} catch (FileNotFoundException e) {
			} catch (IOException e) {
			}
		}
	}
	
	/**
	 * Closes this upload connection.
	 */
	public synchronized void close() {
		isFinishedTransmitting = true;
		running = false;
		notify();
		
		server.getFileTransferManager().closeConnection(this);
		
		try {
			socket.close();
			in.close();
			out.close();
		} catch (Exception e) {
		}
	}

}
