package digital_home;

import java.io.IOException;
import java.net.Socket;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

/**
 * Class representing a connection sending and receiving XML documents.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class XMLConnection extends Connection {

	private SAXBuilder sax;
	private XMLOutputter outputter;
	
	private String version;
	private boolean clientUpdated = false;
	
	/**
	 * Creates a new connection.
	 * 
	 * @param server The server this connection belongs to.
	 * @param s The socket which this connection uses to communicate.
	 */
	public XMLConnection(Server server, Socket s) {
		super(server, s);
		
		sax = new SAXBuilder("org.apache.xerces.parsers.SAXParser");
	    outputter = new XMLOutputter();
	}
	
	/**
	 * Method for sending a JDOM XML document through the connection.
	 * 
	 * @param doc The document to be sent.
	 */
	public void sendXMLDoc(Document doc) {
		server.log("XMLConnection - " + username + 
				": Sending XML document.");
		
		try {
		      outputter.output(doc, out);
		      
		      /* To send to java-client, an EOF is needed. 
		       * A .NET-client will NOT work if an EOF is applied
		       */
		      // out.write(EOFInputStream.EOF);
		 		      
		      out.flush();
		      //server.log("Dokument sent til klient");
		    }
		    catch (IOException ex) {
		    	//TODO: fikse bedre feilhndtering
		      ex.printStackTrace();
		    }
	}
	
	/**
	 * Method for receiving incoming XML documents. All incoming XML documents
	 * are sent to the server.
	 */
	public void run() {
				
		while (running){
			Document document = null;
			
			try {
				document = sax.build(new EOFInputStream(in));
				server.receiveXMLDoc(document, this);
				server.log("XMLConnection - " + username + 
				": Received XML document.");
			} catch (JDOMException e) {
				server.log("XMLConnection - " + username + 
				": Incoming data not an XML document.");
			} catch (IOException e) {
				server.log("XMLConnection - " + username + 
				": Client disconnected. Closeing connection");
				close();
			}			
		}	
	}
	
	/**
	 * Closes this connection as well as the socket and all streams.
	 */
	public void close() {
		running = false;
		try {
			out.close();
			in.close();
			socket.close();
			server.getConnectionManager().closeConnection(this);
			server.getFileTransferManager().closeDownloadConnection(username);
		} catch (IOException e) {
			//TODO: fikse bedre feilhndtering
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns the version of the client software the connected client is using.
	 * 
	 * @return the version of the client software the connected client is using.
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * Sets the version of the client software the connected client is using.
	 * 
	 * @param version the version of the client software the connected client is using.
	 */
	public void setVersion(String version) {
		this.version = version;
	}
	
	/**
	 * Returns whether or not the client have been updated.
	 * 
	 * @return true if the client has been updated, else false.
	 */
	public boolean isClientUpdated() {
		return clientUpdated;
	}
	
	/**
	 * Sets whether or not the client has been updated.
	 * 
	 * @param clientUpdated true if the client has been updated, else false.
	 */
	public void setClientUpdated(boolean clientUpdated) {
		this.clientUpdated = clientUpdated;
	}

}
