package digital_home.parser;

import java.util.Hashtable;

import org.jdom.Document;
import org.jdom.Element;

import digital_home.Server;
import digital_home.XMLConnection;
import digital_home.XMLVariables;

/**
 * A class which distributes incoming XML documents to the correct service parser.
 * 
 * All service parsers have to be registered here.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class ParseManager {
	
	/* The hashtable containing all service parsers. The parsers are hashed
	 * on the service name */
	private Hashtable<String, XMLServiceParser> parsers;
	
	private Server server;
	
	/**
	 * Creates a new parse manager. All new services have to be registered here 
	 * in the manner described in the comment inside the constructor.
	 *
	 */
	public ParseManager(Server server) {
		this.server = server;
		parsers = new Hashtable<String, XMLServiceParser>();
		
		/* Add parsers to the list here. For example, if you want to add a parser of type Service1Parser, 
		 * add the following code: 
		 * 
		 * XMLServiceParser service1 = new Service1Parser(server);
		 * parsers.put(service1.getServiceID(), service1);
		 */
		
		XMLServiceParser systemResponseParser = new ServerSystemResponseParser(server);
		parsers.put(systemResponseParser.getServiceID(), systemResponseParser);
		
		XMLServiceParser fileDownloadParser = new DownloadParser(server);
		parsers.put(fileDownloadParser.getServiceID(), fileDownloadParser);
		
		XMLServiceParser fileUploadParser = new UploadParser(server);
		parsers.put(fileUploadParser.getServiceID(), fileUploadParser);
		
		XMLServiceParser fileListParser = new FileListParser(server);
		parsers.put(fileListParser.getServiceID(), fileListParser);
		
		XMLServiceParser scheduleRecordingParser = new ScheduleTvRecordingParser(server);
		parsers.put(scheduleRecordingParser.getServiceID(), scheduleRecordingParser);
	}
	
	/**
	 * Takes an XML document and a connection as input and distributes
	 * the XML document to the right service. 
	 * 
	 * @param doc The incoming document.
	 * @param con The connection from which the document was received.
	 */
	public Document distributeXMLDoc(Document doc, XMLConnection con) {
		
		Element root = doc.getRootElement();
		String serviceID ="";
		XMLServiceParser parser = null;
		Element body = null;
		
		/* If the XMLConnection is not authed, only Auth messages should be accepted */
		if(!con.isAuthenticated()) {
			/* If the message is an auth message, it is sent to the
			 * ServerSystemREsponseParser */
			if (XMLVariables.AUTH_ROOT.equals(root.getName())) {
				parser = parsers.get(XMLVariables.SERVER_SYSTEM_RESPONSE);
				body = root;
			}
			else {
				server.log("ParseManager: Uautorisert connection sendte annen melding enn autoriseringsmelding");
				return null;
			}
		}
		/* If the XMLConnection is authed and the document is a service message, 
		 * distribute it to the correc service */
		else {
			if (XMLVariables.SERVICE_ROOT.equals(root.getName())) {
				body = root.getChild(XMLVariables.SERVICE_BODY);
			
				serviceID = root.getAttributeValue(XMLVariables.SERVICE_ID);
				
				if (serviceID == null) {
					server.log("ParseManager: serviceID er null");
					return null;
				}
				
				parser = parsers.get(serviceID);
			} else if (XMLVariables.SERVER_SYSTEM_RESPONSE.equals(root.getName())) {
				parser = parsers.get(XMLVariables.SERVER_SYSTEM_RESPONSE);
				body = root;
			}	
		}
		
		if (parser == null || body == null) {
			server.log("ParseManager: parser eksisterer ikke eller body er null");
			return server.createErrorResponse();
		}
		
		return parser.receiveXML(body, con);	
	}
}
