package digital_home.rmi;

import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import digital_home.NetworkVariables;
import digital_home.XMLVariables;
import digital_home.exception.FileListException;

/**
 * Class containing RMI methods common for all services.
 * 
 * @author Andreas W. Bjerkhaug
 * @author ystein Ellingb
 */
public class RMIRemoteCommon {
	
	RMIReceiverInterface receiver;
	
	/**
	 * Creates a new RMIRemoteCommon object, making the connection to the RMIReceiver.
	 */
	public RMIRemoteCommon() {
		System.setSecurityManager(new LiberalSecurityManager());
		
		try {
			receiver = (RMIReceiverInterface) Naming.lookup(NetworkVariables.RMI_URL);
			
		} catch (NotBoundException e) {
			System.err.println("No RMIReceiver found.");
		} catch (Exception e) {
			System.err.println("Unknown error occured: " + e.getMessage());
		}
	}
	
	/**
	 * Checks the online status of the MCE for the user with the given username.
	 * 
	 * @param username the username of the client.
	 * @return true if the MCE is online, else false.
	 */
	public boolean checkStatus(String username) {
		try {
			return receiver.checkStatus(username);
		} catch (RemoteException e) {
			return false;
		}
	}
	
	/**
	 * Requests a list of files from a given client.
	 * 
	 * @param username the username of the user from which to get the file list-
	 * @param requestType type of files.
	 * @return the file list.
	 * @throws FileListException
	 */
	public ArrayList<String> getFileList(String username, String requestType) throws FileListException {
		ArrayList<String> fileList = new ArrayList<String>();
		
		Element requestRoot = new Element(XMLVariables.SERVICE_ROOT);
		Element requestBody = new Element(XMLVariables.SERVICE_BODY);
		Element requestFileType = new Element("fileType");
		Document request = new Document(requestRoot);
		
		Document response;
		Element responseFileListElement;
		List responseFileList;
		
		/* Creating request for file list */
		requestRoot.setAttribute(XMLVariables.SERVICE_USER, username);
		requestRoot.setAttribute(XMLVariables.SERVICE_ID, "FileList");
		requestFileType.setText(requestType);
		requestBody.addContent(requestFileType);
		requestRoot.addContent(requestBody);
		
		/* Getting file list from server */
		try {
			response = receiver.receiveXML(request);
		} catch (RemoteException re) {
			throw new FileListException("Error getting response.");
		}
		
		/* Parse response into an ArrayList */
		try {
			responseFileListElement = response.getRootElement().getChild(XMLVariables.SERVICE_BODY).getChild("fileList");
			responseFileList = responseFileListElement.getChildren();
			
			for (int i = 0; i < responseFileList.size(); i++) {
				Element file = (Element) responseFileList.get(i);
				String fileName = file.getChildText("fileName");
				String fileType = file.getChildText("fileType");
				
				fileList.add(fileName + "." + fileType);
			}
			
		} catch (Exception e) {
			throw new FileListException("Error parsing response.");
		}
		
		return fileList;
	}
}

